import { Command } from 'commander';
import { runBuild } from './build';
import { commaSeparatedList } from './utils';
import { getBuildOptions } from './config/getBuildOptions';
import { VERSION } from './config/paths';
import { test } from './test';
import { lint } from './lint';
import { format } from './format';
import { runWatch } from './watch';
import { DIST_PATH_NAME } from './config/constants';

const program = new Command();
program.version(VERSION);

program
  .command('build')
  .description(
    `
    构建项目。将TypeScript编译成js。

    例子：

    $ build
    $ build --format cjs,es,umd
    $ build --entry src/example.ts
    $ build --target node
  `,
  )
  .option(
    '-f, --format <format>',
    '指定模块格式，默认为cjs,es',
    commaSeparatedList,
    ['cjs', 'es'],
  )
  .option(
    '--entry, -i <entry>',
    '指定入口文件，默认为src/index.ts或者src/index.tsx',
  )
  .option(
    '--target <target>',
    '指定目标环境，可选项为web、node，默认为web',
    'web',
  )
  .option('--name <name>', '指定在UMD中的名称，默认会取package.json中的name')
  .option(
    '--outDir <outDir>',
    `指定输出目录，默认为${DIST_PATH_NAME}`,
    DIST_PATH_NAME,
  )
  .action(async (options: BuildOptions) => {
    await runBuild(getBuildOptions(options));
  });

program
  .command('watch')
  .description(
    `以监听模式启动项目。每当代码发生变化就会重新编译。

  例子：

    $ watch
    $ watch --format cjs,es,umd
    $ watch --entry src/example.ts
    $ watch --target node
  `,
  )
  .option(
    '-f, --format <format>',
    '指定模块格式，默认为cjs,es',
    commaSeparatedList,
    ['cjs', 'es'],
  )
  .option(
    '--entry, -i <entry>',
    '指定入口文件，默认为src/index.ts或者src/index.tsx',
  )
  .option(
    '--target <target>',
    '指定目标环境，可选项为web、node，默认为web',
    'web',
  )
  .option('--name <name>', '指定在UMD中的名称，默认会取package.json中的name')
  .option(
    '--outDir <outDir>',
    `指定输出目录，默认为${DIST_PATH_NAME}`,
    DIST_PATH_NAME,
  )
  .action(async (options: BuildOptions) => {
    await runWatch(getBuildOptions(options));
  });

program.command('lint').description('代码检查');
program.command('test').description('单元测试');
program.command('format').description('格式化代码');

const command = process.argv[2];
if (command === 'test') {
  test();
} else if (command === 'lint') {
  lint();
} else if (command === 'format') {
  format();
} else {
  program.parse(process.argv);
}
